/*
    filters image processing routines

    The contents of this file are subject to the Mozilla Public
    License Version 1.1 (the "License"); you may not use this file
    except in compliance with the License. You may obtain a copy of
    the License at http://www.mozilla.org/MPL/
    
    Software distributed under the License is distributed on an "AS
    IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
    implied. See the License for the specific language governing
    rights and limitations under the License.
   
    The Original Code is the filters.o collection of image processing 
    routines.
    
    The Initial Developer of the Original Code is A. Schiffler.  
    Portions created by A. Schiffler are Copyright (C) 1999-2000
    A. Schiffler.  All Rights Reserved.
    
    Contributor(s): Andreas Schiffler <aschiffler@home.com>
                                                                                                                dnl 
    Alternatively, the contents of this file may be used under the
    terms of the GNU General Public License Version 2 or later (the
    "GPL"), in which case the provisions of the GPL are applicable 
    instead of those above.  If you wish to allow use of your 
    version of this file only under the terms of the GPL and not to
    allow others to use your version of this file under the MPL,
    indicate your decision by deleting the provisions above and
    replace them with the notice and other provisions required by
    the GPL.  If you do not delete the provisions above, a recipient
    may use your version of this file under either the MPL or the
    GPL.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    To contact the author try:
    |  Andreas Schiffler                    aschiffler@home.com  |
    |  Senior Systems Engineer  -  Deskplayer.Com Inc., Buffalo  |
    |  4707 Eastwood Cres., Niagara Falls, Ont  L2E 1B4, Canada  |
    |  +1-905-371-3652 (private)  -  +1-905-371-8834 (work/fax)  |

*/

/* Globals to be used by assembler routines for passing parameters */

unsigned char *asmsrc;
unsigned char *asmsrc1;
unsigned char *asmsrc2;
unsigned char *asmdst;
unsigned char *asmstoppos;
unsigned char *asmlow;

/* List of external assembly routines */

extern void asmabsdiff(void);
extern void asmaddbytes(void);
extern void asmandbytes(void);
extern void asmbinarize(void);
 
/*
 -------- absdiff
 
 Calculates the absolute difference of two fields of unsigned bytes 
 "src1" and "src2" of size "size" and stores result in "dst". 
 Uses MMX optimized assembly routine.
 To get best performance, use fields that are at least dword-aligned.
*/

void absdiff (unsigned char *src1, unsigned char *src2, unsigned char *dst, unsigned int size)
{
 unsigned int i;
 unsigned int vk, nk;
 unsigned char *stoppos;
 
 /* Precalculate variables */ 
 vk=(size >> 3);
 nk=size & 7;
 stoppos=dst;
 stoppos += (vk << 3);

 /* Assign asm-routine variables */
 asmsrc1    = src1;
 asmsrc2    = src2;
 asmdst     = dst;
 asmstoppos = stoppos;

 /* Call asm routine */
 asmabsdiff();
 
 /* Process unaligned bytes */
 if (nk>0){
  for (i=(vk<<3); i<size; i++){
   dst[i]=(unsigned char)abs((int)src1[i]-(int)src2[i]);
  }
 }
}

 
/*
 -------- addbytes
 
 Adds two fields of unsigned bytes "src1" and "src2" of size "size" 
 and stores result in "dst". 
 Uses MMX optimized assembly routine.
 To get best performance, use fields that are at least dword-aligned.
*/

void addbytes (unsigned char *src1, unsigned char *src2, unsigned char *dst, unsigned int size)
{
 unsigned int i;
 unsigned int vk, nk;
 unsigned char *stoppos;
 unsigned short sum;
 
 /* Precalculate variables */ 
 vk=(size >> 3);
 nk=size & 7;
 stoppos=dst;
 stoppos += (vk << 3);

 /* Assign asm-routine variables */
 asmsrc1    = src1;
 asmsrc2    = src2;
 asmdst     = dst;
 asmstoppos = stoppos;

 /* Call asm routine */
 asmaddbytes();
 
 /* Process unaligned bytes */
 if (nk>0){
  for (i=(vk<<3); i<size; i++){
   sum=src1[i]+src2[i];
   if (sum>255) {
    dst[i]=255;
   } else {
    dst[i]=(unsigned char)sum;
   } 
  }
 }
}

/*
 -------- andbytes
 
 Boolean ANDs two fields of unsigned bytes "src1" and "src2" of size "size" 
 and stores result in "dst". 
 Uses MMX optimized assembly routine.
 To get best performance, use fields that are at least dword-aligned.
*/

void andbytes (unsigned char *src1, unsigned char *src2, unsigned char *dst, unsigned int size)
{
 unsigned int i;
 unsigned int vk, nk;
 unsigned char *stoppos;
 
 /* Precalculate variables */ 
 vk=(size >> 3);
 nk=size & 7;
 stoppos=dst;
 stoppos += (vk << 3);

 /* Assign asm-routine variables */
 asmsrc1    = src1;
 asmsrc2    = src2;
 asmdst     = dst;
 asmstoppos = stoppos;

 /* Call asm routine */
 asmandbytes();
 
 /* Process unaligned bytes */
 if (nk>0){
  for (i=(vk<<3); i<size; i++){
   dst[i]=src1[i] & src2[i];
  }
 }
}


/*
 ------- binarize
 
 Binarizes a fields of unsigned bytes "src" of size "size" with the 
 value "low" and stores result in "dst". Sets all pixels with values
 value >= low to 255 and all others to 0.
 Uses MMX optimized assembly routine.
 To get best performance, use fields that are at least dword-aligned.
*/

void binarize (unsigned char *src, unsigned char *dst, unsigned int size, unsigned char low)
{
 unsigned int i;
 unsigned int vk, nk;
 unsigned char *stoppos;
 unsigned int locallow;
   
 /* Precalculate variables */ 
 vk=(size >> 3);
 nk=size & 7;
 stoppos=src;
 stoppos += (vk << 3);
 locallow=(unsigned int)low;
 locallow = locallow | (locallow << 8) | (locallow << 16) | (locallow << 24);
       
 /* Assign asm-routine variables */
 asmsrc     = src;
 asmdst     = dst;
 asmstoppos = stoppos;
 asmlow     = (unsigned char *)locallow;

 /* Call asm routine */
 asmbinarize();

 /* Process unaligned bytes */
 if (nk>0){
  for (i=(vk<<3); i<size; i++){
   if (src[i]>=low) {
    dst[i]=255; 
   } else {
    dst[i]=0;
   } 
  }
 }
}

/*
 ------- threshold
 
 Thresholds a fields of unsigned bytes "src" of size "size" with the 
 value "low" and stores result in "dst". Sets all pixels with values
 value >= low to value and all others to 0.
 Uses MMX optimized assembly routine.
 To get best performance, use fields that are at least dword-aligned.
*/

void threshold (unsigned char *src, unsigned char *dst, unsigned int size, unsigned char low)
{
 unsigned int i;
 unsigned int vk, nk;
 unsigned char *stoppos;
 unsigned int locallow;
   
 /* Precalculate variables */ 
 vk=(size >> 3);
 nk=size & 7;
 stoppos=src;
 stoppos += (vk << 3);
 locallow=(unsigned int)low;
 locallow = locallow | (locallow << 8) | (locallow << 16) | (locallow << 24);
       
 /* Assign asm-routine variables */
 asmsrc     = src;
 asmdst     = dst;
 asmstoppos = stoppos;
 asmlow     = (unsigned char *)locallow;

 /* Call asm routine */
 asmthreshold();

 /* Process unaligned bytes */
 if (nk>0){
  for (i=(vk<<3); i<size; i++){
   if (src[i]>=low) {
    dst[i]=src[i]; 
   } else {
    dst[i]=0;
   } 
  }
 }
}
